@@ -1,63 +1,62 @@
 import datetime
-import os
 import shutil
-import tempfile
 import sqlite3
 
 import pytest
 import vcr
-from pprint import pprint
 
 import johnnycanencrypt as jce
 import johnnycanencrypt.johnnycanencrypt as rjce
 
-from .utils import clean_outputfiles, verify_files
+from .conftest import BASE_TESTSDIR
+from .utils import verify_files
 
 DATA = "Kushal loves 🦀"
 
 
-def setup_module(module):
-    module.tmpdirname = tempfile.TemporaryDirectory()
-
-
-def teardown_module(module):
-    del module.tmpdirname
-
-
 def test_correct_keystore_path():
-    ks = jce.KeyStore("tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
 
 
 def test_nonexisting_keystore_path():
     with pytest.raises(OSError):
-        ks = jce.KeyStore("tests/files2/")
+        ks = jce.KeyStore(BASE_TESTSDIR / "files2/")
+
+
+def test_str(tmp_path):
+    ks = jce.KeyStore(tmp_path)
+    assert str(ks) == f"<KeyStore dbpath={tmp_path.as_posix()}/jce.db>"
 
 
 def test_no_such_key():
     with pytest.raises(jce.KeyNotFoundError):
-        ks = jce.KeyStore("tests/files/store")
+        ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
         key = ks.get_key("A4F388BBB194925AE301F844C52B42177857DD79")
 
-def test_create_primary_key_with_encryption():
-    ks = jce.KeyStore(tmpdirname.name)
-    newkey = ks.create_key("redhat", "test key42 <42@example.com>", jce.Cipher.RSA4k, whichkeys=1, can_primary_sign=True)
+
+def test_create_primary_key_with_encryption(tmp_path):
+    ks = jce.KeyStore(tmp_path.as_posix())
+    newkey = ks.create_key(
+        "redhat",
+        "test key42 <42@example.com>",
+        jce.Cipher.RSA4k,
+        whichkeys=1,
+        can_primary_sign=True,
+    )
     assert newkey.can_primary_sign == True
 
-def test_keystore_lifecycle():
-    # Before anything let us first delete if any existing db
-    pathname = os.path.join(tmpdirname.name, "jce.db")
-    if os.path.exists(pathname):
-        os.remove(pathname)
+
+def test_keystore_lifecycle(tmp_path):
     # Now create a fresh db
-    ks = jce.KeyStore(tmpdirname.name)
+    ks = jce.KeyStore(tmp_path.as_posix())
     newkey = ks.create_key("redhat", "test key1 <email@example.com>", jce.Cipher.RSA4k)
     # the default key must be of secret
     assert newkey.keytype == jce.KeyType.SECRET
 
-    ks.import_key("tests/files/store/public.asc")
-    ks.import_key("tests/files/store/pgp_keys.asc")
-    ks.import_key("tests/files/store/hellopublic.asc")
-    ks.import_key("tests/files/store/secret.asc")
+    ks.import_key((BASE_TESTSDIR / "files/store/public.asc").as_posix())
+    ks.import_key((BASE_TESTSDIR / "files/store/pgp_keys.asc").as_posix())
+    ks.import_key((BASE_TESTSDIR / "files/store/hellopublic.asc").as_posix())
+    ks.import_key((BASE_TESTSDIR / "files/store/secret.asc").as_posix())
     # Now check the numbers of keys in the store
     assert (2, 2) == ks.details()
 
@@ -82,12 +81,14 @@ def test_keystore_lifecycle():
     assert key_via_fingerprint == keys_via_names[0]
 
 
-def test_keystore_contains_key():
+def test_keystore_contains_key(tmp_path):
     "verifies __contains__ method for keystore"
-    ks = jce.KeyStore(tmpdirname.name)
-    keypath = "tests/files/store/secret.asc"
-    k = ks.import_key(keypath)
-    _, fingerprint, keytype, exp, ctime, othervalues = jce.parse_cert_file(keypath)
+    ks = jce.KeyStore(tmp_path.as_posix())
+    keypath = BASE_TESTSDIR / "files/store/secret.asc"
+    k = ks.import_key(keypath.as_posix())
+    _, fingerprint, keytype, exp, ctime, othervalues = jce.parse_cert_file(
+        keypath.as_posix()
+    )
 
     # First only the fingerprint
     assert fingerprint in ks
@@ -98,18 +99,18 @@ def test_keystore_contains_key():
 
 
 def test_keystore_details():
-    ks = jce.KeyStore("./tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     assert (1, 2) == ks.details()
 
 
 def test_keystore_keyids():
-    ks = jce.KeyStore("./tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     key = ks.get_key("A85FF376759C994A8A1168D8D8219C8C43F6C5E1")
     assert key.keyid == "D8219C8C43F6C5E1"
 
 
 def test_keystore_get_via_keyids():
-    ks = jce.KeyStore("./tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     key = ks.get_key("A85FF376759C994A8A1168D8D8219C8C43F6C5E1")
     keys = ks.get_keys_by_keyid("FB82AA5D326DA75D")
     assert len(keys) == 1
@@ -117,28 +118,26 @@ def test_keystore_get_via_keyids():
 
 
 def test_keystore_key_uids():
-    ks = jce.KeyStore("./tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     key = ks.get_key("A85FF376759C994A8A1168D8D8219C8C43F6C5E1")
     assert "kushal@fedoraproject.org" == key.uids[0]["email"]
     assert "mail@kushaldas.in" == key.uids[-1]["email"]
 
 
-def test_key_password_change():
-    tempdir = tempfile.TemporaryDirectory()
-    ks = jce.KeyStore(tempdir.name)
-    k = ks.import_key("tests/files/store/secret.asc")
+def test_key_password_change(tmp_path):
+    ks = jce.KeyStore(tmp_path.as_posix())
+    k = ks.import_key((BASE_TESTSDIR / "files/store/secret.asc").as_posix())
     k2 = ks.update_password(k, "redhat", "byebye")
     data = ks.sign_detached(k2, b"hello", "byebye")
 
 
-def test_key_deletion():
-    tempdir = tempfile.TemporaryDirectory()
-    ks = jce.KeyStore(tempdir.name)
-    ks.import_key("tests/files/store/public.asc")
-    k = ks.import_key("tests/files/store/pgp_keys.asc")
-    ks.import_key("tests/files/store/hellopublic.asc")
-    ks.import_key("tests/files/store/hellosecret.asc")
-    ks.import_key("tests/files/store/secret.asc")
+def test_key_deletion(tmp_path):
+    ks = jce.KeyStore(tmp_path.as_posix())
+    ks.import_key((BASE_TESTSDIR / "files/store/public.asc").as_posix())
+    k = ks.import_key((BASE_TESTSDIR / "files/store/pgp_keys.asc").as_posix())
+    ks.import_key((BASE_TESTSDIR / "files/store/hellopublic.asc").as_posix())
+    ks.import_key((BASE_TESTSDIR / "files/store/hellosecret.asc").as_posix())
+    ks.import_key((BASE_TESTSDIR / "files/store/secret.asc").as_posix())
     assert (1, 2) == ks.details()
 
     ks.delete_key("F4F388BBB194925AE301F844C52B42177857DD79")
@@ -156,17 +155,16 @@ def test_key_deletion():
 
 
 def test_key_equality():
-    ks = jce.KeyStore("tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
     assert key.fingerprint == "F51C310E02DC1B7771E176D8A1C5C364EB5B9A20"
 
 
-def test_ks_update_expiry_time_for_subkeys():
+def test_ks_update_expiry_time_for_subkeys(tmp_path):
     "Updates expiry time for a given subkey"
-    tempdir = tempfile.TemporaryDirectory()
-    ks = jce.KeyStore(tempdir.name)
-    ks.import_key("tests/files/store/hellosecret.asc")
-    ks.import_key("tests/files/store/secret.asc")
+    ks = jce.KeyStore(tmp_path.as_posix())
+    ks.import_key((BASE_TESTSDIR / "files/store/hellosecret.asc").as_posix())
+    ks.import_key((BASE_TESTSDIR / "files/store/secret.asc").as_posix())
 
     key = ks.get_key("F4F388BBB194925AE301F844C52B42177857DD79")
     subkeys = [
@@ -185,7 +183,7 @@ def test_ks_update_expiry_time_for_subkeys():
 
 def test_ks_encrypt_decrypt_bytes():
     "Encrypts and decrypt some bytes"
-    ks = jce.KeyStore("tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     public_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
     encrypted = ks.encrypt(public_key, DATA)
     assert encrypted.startswith(b"-----BEGIN PGP MESSAGE-----\n")
@@ -198,7 +196,7 @@ def test_ks_encrypt_decrypt_bytes():
 
 def test_ks_encrypt_decrypt_bytes_multiple_recipients():
     "Encrypts and decrypt some bytes"
-    ks = jce.KeyStore("tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     key1 = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
     key2 = ks.get_key("F4F388BBB194925AE301F844C52B42177857DD79")
     encrypted = ks.encrypt([key1, key2], DATA)
@@ -215,12 +213,12 @@ def test_ks_encrypt_decrypt_bytes_multiple_recipients():
     assert DATA == decrypted_text
 
 
-def test_ks_encrypt_decrypt_bytes_to_file():
+def test_ks_encrypt_decrypt_bytes_to_file(tmp_path):
     "Encrypts and decrypt some bytes"
-    outputfile = os.path.join(tmpdirname.name, "encrypted.asc")
-    ks = jce.KeyStore("tests/files/store")
+    outputfile = tmp_path / "encrypted.asc"
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     secret_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
-    assert ks.encrypt(secret_key, DATA, outputfile=outputfile)
+    assert ks.encrypt(secret_key, DATA, outputfile=outputfile.as_posix())
     with open(outputfile, "rb") as fobj:
         encrypted = fobj.read()
     secret_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
@@ -230,13 +228,13 @@ def test_ks_encrypt_decrypt_bytes_to_file():
     assert DATA == decrypted_text
 
 
-def test_ks_encrypt_decrypt_bytes_to_file_multiple_recipients():
+def test_ks_encrypt_decrypt_bytes_to_file_multiple_recipients(tmp_path):
     "Encrypts and decrypt some bytes"
-    outputfile = os.path.join(tmpdirname.name, "encrypted.asc")
-    ks = jce.KeyStore("tests/files/store")
+    outputfile = tmp_path / "encrypted.asc"
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     key1 = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
     key2 = ks.get_key("F4F388BBB194925AE301F844C52B42177857DD79")
-    assert ks.encrypt([key1, key2], DATA, outputfile=outputfile)
+    assert ks.encrypt([key1, key2], DATA, outputfile=outputfile.as_posix())
     with open(outputfile, "rb") as fobj:
         encrypted = fobj.read()
     secret_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
@@ -246,56 +244,56 @@ def test_ks_encrypt_decrypt_bytes_to_file_multiple_recipients():
     assert DATA == decrypted_text
 
 
-def test_ks_encrypt_decrypt_file(encrypt_decrypt_file):
+def test_ks_encrypt_decrypt_file(tmp_path):
     "Encrypts and decrypt some bytes"
-    inputfile = "tests/files/text.txt"
-    output = "/tmp/text-encrypted.pgp"
-    decrypted_output = "/tmp/text.txt"
+    inputfile = BASE_TESTSDIR / "files/text.txt"
+    output = tmp_path / "text-encrypted.pgp"
+    decrypted_output = tmp_path / "text.txt"
 
-    ks = jce.KeyStore("tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     public_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
-    assert ks.encrypt_file(public_key, inputfile, output)
+    assert ks.encrypt_file(public_key, inputfile.as_posix(), output.as_posix())
     secret_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
-    ks.decrypt_file(secret_key, output, decrypted_output, password="redhat")
+    ks.decrypt_file(secret_key, output.as_posix(), decrypted_output.as_posix(), password="redhat")
     verify_files(inputfile, decrypted_output)
 
 
-def test_ks_encrypt_decrypt_filehandler(encrypt_decrypt_file):
+def test_ks_encrypt_decrypt_filehandler(tmp_path):
     "Encrypts and decrypt some bytes"
-    inputfile = "tests/files/text.txt"
-    output = "/tmp/text-encrypted.pgp"
-    decrypted_output = "/tmp/text.txt"
+    inputfile = BASE_TESTSDIR / "files/text.txt"
+    output = tmp_path / "text-encrypted.pgp"
+    decrypted_output = tmp_path / "text.txt"
 
-    ks = jce.KeyStore("tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     public_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
     with open(inputfile, "rb") as fobj:
-        assert ks.encrypt_file(public_key, fobj, output)
+        assert ks.encrypt_file(public_key, fobj, output.as_posix())
     secret_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
     with open(output, "rb") as fobj:
-        ks.decrypt_file(secret_key, fobj, decrypted_output, password="redhat")
+        ks.decrypt_file(secret_key, fobj, decrypted_output.as_posix(), password="redhat")
     verify_files(inputfile, decrypted_output)
 
 
-def test_ks_encrypt_decrypt_file_multiple_recipients(encrypt_decrypt_file):
+def test_ks_encrypt_decrypt_file_multiple_recipients(tmp_path):
     "Encrypts and decrypt some bytes"
-    inputfile = "tests/files/text.txt"
-    output = "/tmp/text-encrypted.pgp"
-    decrypted_output = "/tmp/text.txt"
+    inputfile = BASE_TESTSDIR / "files/text.txt"
+    output = tmp_path / "text-encrypted.pgp"
+    decrypted_output = tmp_path / "text.txt"
 
-    ks = jce.KeyStore("tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     key1 = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
     key2 = ks.get_key("F4F388BBB194925AE301F844C52B42177857DD79")
-    encrypted = ks.encrypt_file([key1, key2], inputfile, output)
+    encrypted = ks.encrypt_file([key1, key2], inputfile.as_posix(), output.as_posix())
     secret_key1 = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
-    ks.decrypt_file(secret_key1, output, decrypted_output, password="redhat")
+    ks.decrypt_file(secret_key1, output.as_posix(), decrypted_output.as_posix(), password="redhat")
     verify_files(inputfile, decrypted_output)
     secret_key2 = ks.get_key("F4F388BBB194925AE301F844C52B42177857DD79")
-    ks.decrypt_file(secret_key2, output, decrypted_output, password="redhat")
+    ks.decrypt_file(secret_key2, output.as_posix(), decrypted_output.as_posix(), password="redhat")
     verify_files(inputfile, decrypted_output)
 
 
 def test_ks_sign_data():
-    ks = jce.KeyStore("tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     key = "F51C310E02DC1B7771E176D8A1C5C364EB5B9A20"
     signed = ks.sign_detached(key, "hello", "redhat")
     assert signed.startswith("-----BEGIN PGP SIGNATURE-----\n")
@@ -303,37 +301,69 @@ def test_ks_sign_data():
 
 
 def test_ks_sign_data_fails():
-    ks = jce.KeyStore("tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     key = "F51C310E02DC1B7771E176D8A1C5C364EB5B9A20"
     signed = ks.sign_detached(key, "hello", "redhat")
     assert signed.startswith("-----BEGIN PGP SIGNATURE-----\n")
     assert not ks.verify(key, "hello2", signed)
 
 
-def test_ks_sign_verify_file_detached():
-    inputfile = "tests/files/text.txt"
-    tempdir = tempfile.TemporaryDirectory()
-    shutil.copy(inputfile, tempdir.name)
-    ks = jce.KeyStore("tests/files/store")
+def test_ks_sign_verify_file_detached(tmp_path):
+    inputfile = BASE_TESTSDIR / "files/text.txt"
+    shutil.copy(inputfile, tmp_path.as_posix())
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     key = "F51C310E02DC1B7771E176D8A1C5C364EB5B9A20"
-    file_to_be_signed = os.path.join(tempdir.name, "text.txt")
-    signed = ks.sign_file_detached(key, file_to_be_signed, "redhat", write=True)
+    file_to_be_signed = tmp_path / "text.txt"
+    signed = ks.sign_file_detached(key, file_to_be_signed.as_posix(), "redhat", write=True)
     assert signed.startswith("-----BEGIN PGP SIGNATURE-----\n")
-    assert ks.verify_file_detached(key, file_to_be_signed, file_to_be_signed + ".asc")
+    assert ks.verify_file_detached(key, file_to_be_signed.as_posix(), file_to_be_signed.as_posix() + ".asc")
 
 
-def test_ks_creation_expiration_time():
+def test_ks_userid_signing(tmp_path):
+    # Now create a fresh db
+    ks = jce.KeyStore(tmp_path.as_posix())
+    k = ks.import_key((BASE_TESTSDIR / "files/store/pgp_keys.asc").as_posix())
+    t2 = ks.import_key((BASE_TESTSDIR / "files/store/secret.asc").as_posix())
+
+    # now let us sign the keys in kushal's uids
+    k = ks.certify_key(
+        t2,
+        k,
+        ["Kushal Das <kushaldas@gmail.com>", "Kushal Das <kushal@fedoraproject.org>"],
+        jce.SignatureType.PersonaCertification,
+        password="redhat",
+    )
+    # k now contains the new updated key
+    for uid in k.uids:
+        if (
+            uid["value"] == "Kushal Das <kushaldas@gmail.com>"
+            or uid["value"] == "Kushal Das <kushal@fedoraproject.org>"
+        ):
+            certs = uid["certifications"]
+            # Only the new certification
+            assert len(certs) == 1
+            cert = certs[0]
+            assert cert["certification_type"] == "persona"
+            for data in cert["certification_list"]:
+                if data[0] == "fingerprint":
+                    assert data[1] == "F4F388BBB194925AE301F844C52B42177857DD79"
+                if data[0] == "keyid":
+                    assert data[1] == "C52B42177857DD79"
+        else:
+            assert len(uid["certifications"]) == 0
+
+
+def test_ks_creation_expiration_time(tmp_path):
     """
     Tests via Kushal's key and a new key
     """
     # These two are known values from kushal
     etime = datetime.datetime(2020, 10, 16, 20, 53, 47)
     ctime = datetime.datetime(2017, 10, 17, 20, 53, 47)
-    tmpdir = tempfile.TemporaryDirectory()
     # First let us check from the file
-    keypath = "tests/files/store/pgp_keys.asc"
-    ks = jce.KeyStore(tmpdir.name)
-    k = ks.import_key(keypath)
+    keypath = BASE_TESTSDIR / "files/store/pgp_keys.asc"
+    ks = jce.KeyStore(tmp_path.as_posix())
+    k = ks.import_key(keypath.as_posix())
     assert etime.date() == k.expirationtime.date()
     assert ctime.date() == k.creationtime.date()
 
@@ -345,10 +375,10 @@ def test_ks_creation_expiration_time():
     assert ctime.date() == newk.creationtime.date()
     assert not newk.expirationtime
 
-    # Now both creation and expirationtime
+    # Now both creation and expirationtime for primary key
     ctime = datetime.datetime(2008, 10, 10, 20, 53, 47)
     etime = datetime.datetime(2025, 12, 15, 20, 53, 47)
-    newk = ks.create_key("redhat", "Another test key", creation=ctime, expiration=etime)
+    newk = ks.create_key("redhat", "Another test key", creation=ctime, expiration=etime, can_primary_expire=True)
     assert ctime.date() == newk.creationtime.date()
     assert etime.date() == newk.expirationtime.date()
 
@@ -375,12 +405,26 @@ def test_ks_creation_expiration_time():
         subkeys_expiration=True,
     )
     assert datetime.datetime.now().date() == newk.creationtime.date()
+    assert not newk.expirationtime
     for skeyid, subkey in newk.othervalues["subkeys"].items():
         assert subkey[1].date() == etime.date()
 
+    # Now verify both subkeys and primary can expire
+    etime = datetime.datetime(2030, 6, 5, 20, 53, 47)
+    newk = ks.create_key(
+        "redhat",
+        "Test key with subkey expiration",
+        expiration=etime,
+        subkeys_expiration=True,
+        can_primary_expire=True,
+    )
+    assert datetime.datetime.now().date() == newk.creationtime.date()
+    assert etime.date() == newk.expirationtime.date()
+    for skeyid, subkey in newk.othervalues["subkeys"].items():
+        assert subkey[1].date() == etime.date()
 
 def test_get_all_keys():
-    ks = jce.KeyStore("./tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     keys = ks.get_all_keys()
     assert 3 == len(keys)
     # TODO: add more checks here in future
@@ -388,22 +432,21 @@ def test_get_all_keys():
 
 def test_get_pub_key():
     """Verifies that we export only the public key part from any key"""
-    ks = jce.KeyStore("./tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     fingerprint = "F51C310E02DC1B7771E176D8A1C5C364EB5B9A20"
     key = ks.get_key(fingerprint)
     # verify that the key is a secret
-    key.keytype == 1
+    assert key.keytype == jce.KeyType.SECRET
 
     # now get the public key
     pub_key = key.get_pub_key()
     assert pub_key.startswith("-----BEGIN PGP PUBLIC KEY BLOCK-----")
 
 
-def test_add_userid():
+def test_add_userid(tmp_path):
     """Verifies that we can add uid to a cert"""
-    tempdir = tempfile.TemporaryDirectory()
-    ks = jce.KeyStore(tempdir.name)
-    key = ks.import_key("tests/files/store/secret.asc")
+    ks = jce.KeyStore(tmp_path.as_posix())
+    key = ks.import_key((BASE_TESTSDIR / "files/store/secret.asc").as_posix())
     # check that there is only one userid
     assert len(key.uids) == 1
 
@@ -415,11 +458,10 @@ def test_add_userid():
     assert key2.keytype == jce.KeyType.SECRET
 
 
-def test_add_and_revoke_userid():
+def test_add_and_revoke_userid(tmp_path):
     """Verifies that we can add uid to a cert"""
-    tempdir = tempfile.TemporaryDirectory()
-    ks = jce.KeyStore(tempdir.name)
-    key = ks.import_key("tests/files/store/secret.asc")
+    ks = jce.KeyStore(tmp_path.as_posix())
+    key = ks.import_key((BASE_TESTSDIR / "files/store/secret.asc").as_posix())
     # check that there is only one userid
     assert len(key.uids) == 1
 
@@ -446,11 +488,10 @@ def test_add_and_revoke_userid():
             assert uid["revoked"] == False
 
 
-def test_add_userid_fails_for_public():
+def test_add_userid_fails_for_public(tmp_path):
     """Verifies that adding uid to a public key fails"""
-    tempdir = tempfile.TemporaryDirectory()
-    ks = jce.KeyStore(tempdir.name)
-    key = ks.import_key("tests/files/store/public.asc")
+    ks = jce.KeyStore(tmp_path.as_posix())
+    key = ks.import_key((BASE_TESTSDIR / "files/store/public.asc").as_posix())
     # verify that the key is a secret
     assert len(key.uids) == 1
 
@@ -461,7 +502,7 @@ def test_add_userid_fails_for_public():
 
 def test_update_subkey_expiry_time():
     "Updates the expirytime for a given subkey"
-    ks = jce.KeyStore("tests/files/store")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
     key = ks.get_key("F4F388BBB194925AE301F844C52B42177857DD79")
     fps = [
         "102EBD23BD5D2D340FBBDE0ADFD1C55926648D2F",
@@ -477,25 +518,22 @@ def test_update_subkey_expiry_time():
             assert date.date() == tomorrow
 
 
-def test_same_key_import_error():
-    tempdir = tempfile.TemporaryDirectory()
-    ks = jce.KeyStore(tempdir.name)
-    ks.import_key("tests/files/store/public.asc")
+def test_same_key_import_error(tmp_path):
+    ks = jce.KeyStore(tmp_path.as_posix())
+    ks.import_key((BASE_TESTSDIR / "files/store/public.asc").as_posix())
     with pytest.raises(jce.CryptoError):
-        ks.import_key("tests/files/store/public.asc")
+        ks.import_key((BASE_TESTSDIR / "files/store/public.asc").as_posix())
 
 
-def test_key_without_uid():
-    tempdir = tempfile.TemporaryDirectory()
-    ks = jce.KeyStore(tempdir.name)
+def test_key_without_uid(tmp_path):
+    ks = jce.KeyStore(tmp_path.as_posix())
     k = ks.create_key("redhat")
     uids, fp, secret, et, ct, othervalues = jce.parse_cert_bytes(k.keyvalue)
     assert len(uids) == 0
 
 
-def test_key_with_multiple_uids():
-    tempdir = tempfile.TemporaryDirectory()
-    ks = jce.KeyStore(tempdir.name)
+def test_key_with_multiple_uids(tmp_path):
+    ks = jce.KeyStore(tmp_path.as_posix())
     uids = [
         "Kushal Das <kushaldas@gmail.com>",
         "kushal@freedom.press",
@@ -506,11 +544,11 @@ def test_key_with_multiple_uids():
     assert len(uids) == 3
 
 
-def test_ks_upgrade():
+def test_ks_upgrade(tmp_path):
     "tests db upgrade from an old db"
-    tempdir = tempfile.TemporaryDirectory()
-    shutil.copy("tests/files/store/oldjce.db", os.path.join(tempdir.name, "jce.db"))
-    ks = jce.KeyStore(tempdir.name)
+    shutil.copy(BASE_TESTSDIR / "files/store/oldjce.db",  tmp_path / "jce.db")
+
+    ks = jce.KeyStore(tmp_path.as_posix())
     con = sqlite3.connect(ks.dbpath)
     con.row_factory = sqlite3.Row
     # First we will check if this db schema is old or not
@@ -520,33 +558,34 @@ def test_ks_upgrade():
         cursor.execute(sql)
         fromdb = cursor.fetchone()
         assert fromdb["upgradedate"] == jce.DB_UPGRADE_DATE
+    # TODO: Now verify the keys inside of the new db, in full.
 
 
-def test_ks_upgrade_failure():
+def test_ks_upgrade_failure(tmp_path):
     "tests db upgrade failure from an old db because of existing file"
-    tempdir = tempfile.TemporaryDirectory()
-    shutil.copy("tests/files/store/oldjce.db", os.path.join(tempdir.name, "jce.db"))
+    shutil.copy(BASE_TESTSDIR / "files/store/oldjce.db", tmp_path / "jce.db")
     shutil.copy(
-        "tests/files/store/oldjce.db", os.path.join(tempdir.name, "jce_upgrade.db")
+        BASE_TESTSDIR / "files/store/oldjce.db", tmp_path / "jce_upgrade.db"
     )
     with pytest.raises(RuntimeError):
-        ks = jce.KeyStore(tempdir.name)
+        ks = jce.KeyStore(tmp_path.as_posix())
 
 
 def test_get_encrypted_for():
-    ks = jce.KeyStore("tests/files/store/")
-    keyids = rjce.file_encrypted_for("tests/files/double_recipient.asc")
+    ks = jce.KeyStore(BASE_TESTSDIR / "files/store/")
+    keyids = rjce.file_encrypted_for(
+        (BASE_TESTSDIR / "files/double_recipient.asc").as_posix())
     assert keyids == ["1CF980B8E69E112A", "5A7A1560D46ED4F6"]
-    with open("tests/files/double_recipient.asc", "rb") as fobj:
+    with open(BASE_TESTSDIR / "files/double_recipient.asc", "rb") as fobj:
         data = fobj.read()
     keyids = rjce.bytes_encrypted_for(data)
     assert keyids == ["1CF980B8E69E112A", "5A7A1560D46ED4F6"]
 
 
-@vcr.use_cassette("tests/files/test_fetch_key_by_fingerprint.yml")
-def test_fetch_key_by_fingerprint():
-    tempdir = tempfile.TemporaryDirectory()
-    ks = jce.KeyStore(tempdir.name)
+@vcr.use_cassette(
+    (BASE_TESTSDIR / "files/test_fetch_key_by_fingerprint.yml").as_posix())
+def test_fetch_key_by_fingerprint(tmp_path):
+    ks = jce.KeyStore(tmp_path.as_posix())
     key = ks.fetch_key_by_fingerprint("EF6E286DDA85EA2A4BA7DE684E2C6E8793298290")
     assert len(key.uids) == 1
     uid = key.uids[0]
@@ -554,18 +593,17 @@ def test_fetch_key_by_fingerprint():
     assert uid["name"] == "Tor Browser Developers"
 
 
-@vcr.use_cassette("tests/files/test_fetch_nonexistingkey_by_fingerprint.yml")
-def test_fetch_nonexistingkey_by_fingerprint():
-    tempdir = tempfile.TemporaryDirectory()
-    ks = jce.KeyStore(tempdir.name)
+@vcr.use_cassette(
+    (BASE_TESTSDIR / "files/test_fetch_nonexistingkey_by_fingerprint.yml").as_posix())
+def test_fetch_nonexistingkey_by_fingerprint(tmp_path):
+    ks = jce.KeyStore(tmp_path.as_posix())
     with pytest.raises(jce.KeyNotFoundError):
         key = ks.fetch_key_by_fingerprint("EF6E286DDA85EA2A4BA7DE684E2C6E8793298291")
 
 
-@vcr.use_cassette("tests/files/test_fetch_key_by_email.yml")
-def test_fetch_key_by_email():
-    tempdir = tempfile.TemporaryDirectory()
-    ks = jce.KeyStore(tempdir.name)
+@vcr.use_cassette((BASE_TESTSDIR / "files/test_fetch_key_by_email.yml").as_posix())
+def test_fetch_key_by_email(tmp_path):
+    ks = jce.KeyStore(tmp_path.as_posix())
     key = ks.fetch_key_by_email("anwesha.srkr@gmail.com")
     assert len(key.uids) == 2
     uid = key.uids[0]
@@ -573,9 +611,9 @@ def test_fetch_key_by_email():
     assert key.fingerprint == "2871635BE3B4E5C04F02B848C353BFE051D06C33"
 
 
-@vcr.use_cassette("tests/files/test_fetch_nonexistingkey_by_email.yml")
-def test_fetch_nonexistingkey_by_email():
-    tempdir = tempfile.TemporaryDirectory()
-    ks = jce.KeyStore(tempdir.name)
+@vcr.use_cassette(
+    (BASE_TESTSDIR / "files/test_fetch_nonexistingkey_by_email.yml").as_posix())
+def test_fetch_nonexistingkey_by_email(tmp_path):
+    ks = jce.KeyStore(tmp_path.as_posix())
     with pytest.raises(jce.KeyNotFoundError):
         ks.fetch_key_by_email("doesnotexists@kushaldas.in")
